@@ -171,7 +171,7 @@ class BartForSequenceClassification(Module):
         model.eval()
         for p in model.parameters():
             p.requires_grad = False
-        return model
+        return model.half()
 
     def forward(self, input_ids: Tensor, attention_mask: Tensor) -> Tensor:
         tensor = self.transformer(input_ids, attention_mask=attention_mask)
