@@ -104,8 +104,7 @@ class CoreferencePredictor(torch.nn.Module):
         return model
 
     def forward(self, spans, token_ids, mask, offsets, segment_concat_mask):
-        text_embeddings = self.transformer(
-            token_ids=token_ids, offsets=offsets, segment_concat_mask=segment_concat_mask)
+        text_embeddings = self.transformer(token_ids=token_ids, offsets=offsets, segment_concat_mask=segment_concat_mask)
 
         batch_size, num_spans = spans.shape[:2]
         document_length = text_embeddings.shape[1]
@@ -187,54 +186,51 @@ class CoreferencePredictor(torch.nn.Module):
             top_antecedent_offsets,
         )
 
-        for _ in range(self._inference_order - 1):
-            dummy_mask = torch.ones(
-                (batch_size, num_spans_to_keep, 1), dtype=torch.bool, device=top_antecedent_mask.device)
-            # Shape: (batch_size, num_spans_to_keep, 1 + max_antecedents,)
-            top_antecedent_with_dummy_mask = torch.cat([dummy_mask, top_antecedent_mask], -1)
-            # Shape: (batch_size, num_spans_to_keep, 1 + max_antecedents)
-            attention_weight = masked_softmax(
-                coreference_scores, top_antecedent_with_dummy_mask
-            )
-            # Shape: (batch_size, num_spans_to_keep, 1 + max_antecedents, embedding_size)
-            top_antecedent_with_dummy_embeddings = torch.cat(
-                [top_span_embeddings.unsqueeze(2), top_antecedent_embeddings], 2
-            )
-            # Shape: (batch_size, num_spans_to_keep, embedding_size)
-            attended_embeddings = weighted_sum(top_antecedent_with_dummy_embeddings, attention_weight)
-            # Shape: (batch_size, num_spans_to_keep, embedding_size)
-            top_span_embeddings = self._span_updating_gated_sum(top_span_embeddings, attended_embeddings)
+        dummy_mask = torch.ones(
+            (batch_size, num_spans_to_keep, 1), dtype=torch.bool, device=top_antecedent_mask.device)
+        # Shape: (batch_size, num_spans_to_keep, 1 + max_antecedents,)
+        top_antecedent_with_dummy_mask = torch.cat([dummy_mask, top_antecedent_mask], -1)
+        # Shape: (batch_size, num_spans_to_keep, 1 + max_antecedents)
+        attention_weight = masked_softmax(
+            coreference_scores, top_antecedent_with_dummy_mask
+        )
+        # Shape: (batch_size, num_spans_to_keep, 1 + max_antecedents, embedding_size)
+        top_antecedent_with_dummy_embeddings = torch.cat(
+            [top_span_embeddings.unsqueeze(2), top_antecedent_embeddings], 2
+        )
+        # Shape: (batch_size, num_spans_to_keep, embedding_size)
+        attended_embeddings = weighted_sum(top_antecedent_with_dummy_embeddings, attention_weight)
+        # Shape: (batch_size, num_spans_to_keep, embedding_size)
+        top_span_embeddings = self._span_updating_gated_sum(top_span_embeddings, attended_embeddings)
 
-            # Shape: (batch_size, num_spans_to_keep, max_antecedents, embedding_size)
-            top_antecedent_embeddings = batched_index_select(
-                top_span_embeddings, top_antecedent_indices, flat_top_antecedent_indices
-            )
-            # Shape: (batch_size, num_spans_to_keep, 1 + max_antecedents)
-            coreference_scores = self._compute_coreference_scores(
-                top_span_embeddings,
-                top_antecedent_embeddings,
-                top_partial_coreference_scores,
-                top_antecedent_mask,
-                top_antecedent_offsets
-            )
+        # Shape: (batch_size, num_spans_to_keep, max_antecedents, embedding_size)
+        top_antecedent_embeddings = batched_index_select(
+            top_span_embeddings, top_antecedent_indices, flat_top_antecedent_indices
+        )
+        # Shape: (batch_size, num_spans_to_keep, 1 + max_antecedents)
+        coreference_scores = self._compute_coreference_scores(
+            top_span_embeddings,
+            top_antecedent_embeddings,
+            top_partial_coreference_scores,
+            top_antecedent_mask,
+            top_antecedent_offsets
+        )
 
-            # We now have, for each span which survived the pruning stage,
-            # a predicted antecedent. This implies a clustering if we group
-            # mentions which refer to each other in a chain.
-            # Shape: (batch_size, num_spans_to_keep)
-            _, predicted_antecedents = coreference_scores.max(2)
-            # Subtract one here because index 0 is the "no antecedent" class,
-            # so this makes the indices line up with actual spans if the prediction
-            # is greater than -1.
-            predicted_antecedents -= 1
+        # We now have, for each span which survived the pruning stage,
+        # a predicted antecedent. This implies a clustering if we group
+        # mentions which refer to each other in a chain.
+        # Shape: (batch_size, num_spans_to_keep)
+        _, predicted_antecedents = coreference_scores.max(2)
+        # Subtract one here because index 0 is the "no antecedent" class,
+        # so this makes the indices line up with actual spans if the prediction
+        # is greater than -1.
+        predicted_antecedents -= 1
 
-            output_dict = {
-                'top_spans': top_spans.detach().cpu(),
-                'antecedent_indices': top_antecedent_indices.detach().cpu(),
-                'predicted_antecedents': predicted_antecedents.detach().cpu()
-            }
-
-            return output_dict
+        return {
+            'top_spans': top_spans.detach().cpu(),
+            'antecedent_indices': top_antecedent_indices.detach().cpu(),
+            'predicted_antecedents': predicted_antecedents.detach().cpu()
+        }
 
     def _coarse_to_fine_pruning(
         self,
