@@ -80,7 +80,7 @@ class ElectraForSequenceClassification(Module):
         model.eval()
         for p in model.parameters():
             p.requires_grad = False
-        return model
+        return model.half()
 
     def load_state_dict(self, state_dict, strict=True):
         del state_dict['electra.embeddings.position_ids']
