@@ -3,6 +3,7 @@ from typing import Any, Optional, Union, Tuple, List, Dict
 import time
 import select
 import socket
+import io
 import threading
 import operator
 import uuid
@@ -31,7 +32,9 @@ class ClientSocket(SocketConnection):
 
     def send_inputs(self, data: Any, batch_ind: int):
         self.message_id = uuid.uuid4().bytes[:Config.message_id_len]
-        self.send_message(self.message_id, data)
+        buffer = io.BytesIO()
+        torch.save(data, buffer)
+        self.send_data(self.message_id + buffer.getvalue())
         self.batch_ind = batch_ind
 
     def receive_outputs(self) -> Optional[Tuple[int, Any]]:
@@ -52,18 +55,20 @@ class ClientAPI:
 
     def __init__(self, model: str, model_config: ModelDeploymentConfig):
         self.model = model
-        self.model_config = model_config
+        self.num_models = len(model_config.allocations)
+        self.port = model_config.port
+        self.max_batch_size = model_config.max_batch_size
 
     @staticmethod
     def create_sockets(
-            model: str, model_config: ModelDeploymentConfig, attempts: int, wait_time: float) -> List[ClientSocket]:
+            model: str, num_models: int, port: int, attempts: int, wait_time: float) -> List[ClientSocket]:
         logger.debug('Connecting to model workers')
         model_sockets: List[ClientSocket] = []
-        for worker_ind in range(len(model_config.allocations)):
+        for worker_ind in range(num_models):
             client_socket = socket.socket()
             for _ in range(attempts):
                 try:
-                    client_socket.connect((Config.host, model_config.port + worker_ind))
+                    client_socket.connect((Config.host, port + worker_ind))
                     logger.debug(f'{model} model worker {worker_ind} connected')
                     model_sockets.append(
                         ClientSocket(connection=client_socket, message_size=Config.client_message_size))
@@ -82,6 +87,14 @@ class ClientAPI:
             for ind in range(0, batch_size, max_batch_size)
         ]
 
+    @staticmethod
+    def split_batch(inputs: Dict, batch_size: int) -> Dict:
+        outputs = {}
+        for key, values in inputs.items():
+            outputs[key] = values[:batch_size]
+            inputs[key] = values[batch_size:]
+        return outputs
+
     @staticmethod
     def concat_lists_or_tensors(items: Union[List[List], List[torch.Tensor]]) -> Union[List, torch.Tensor]:
         return (
@@ -110,76 +123,84 @@ class ClientAPI:
                 client_socket.connection.recv(client_socket.message_size)
 
     def forward(self, **inputs):
-        # logger.info('CLIENT FORWARD')
         thread_id = threading.get_ident().to_bytes(length=Config.thread_id_len, byteorder='big', signed=False)
 
         if thread_id not in self.client_sockets:
             self.client_sockets[thread_id] = {}
         if self.model not in self.client_sockets[thread_id]:
             self.client_sockets[thread_id][self.model] = self.create_sockets(
-                self.model, self.model_config, attempts=120, wait_time=10)
+                self.model, self.num_models, self.port, attempts=120, wait_time=10)
 
         client_sockets = self.client_sockets[thread_id][self.model]
 
         self.clear_sockets(client_sockets)
 
-        batched_inputs = self.split_inputs_to_batches(inputs, self.model_config.max_batch_size)
+        total_batch_size = next(iter(inputs.values())).shape[0]
 
-        outputs: List = [None] * len(batched_inputs)
+        outputs: List = []
         received_counter = 0
         cur_ind = 0
 
         for client_socket in client_sockets:
             client_socket.request_server()
 
-        while received_counter < len(outputs):
-            # logger.info('SELECTING')
+        while total_batch_size or received_counter < len(outputs):
             ready_sockets: List[ClientSocket] = select.select(
-                client_sockets if batched_inputs else
+                client_sockets if total_batch_size else
                 [client_socket for client_socket in client_sockets if client_socket.message_id is not None],
                 [], [], None
             )[0]
-            # logger.info('SELECTED')
             input_sockets: List[ClientSocket] = []
             output_sockets: List[ClientSocket] = []
             for client_socket in ready_sockets:
                 (input_sockets if client_socket.message_id is None else output_sockets).append(client_socket)
 
-            added_connections = min(len(input_sockets), len(batched_inputs))
+            added_connections = min(len(input_sockets), total_batch_size)
+            outputs.extend(None for _ in range(added_connections))
+
+            if added_connections:
+                preferred_batch_size = (total_batch_size - 1) // added_connections + 1
+                if preferred_batch_size > self.max_batch_size:
+                    total_batch_size -= self.max_batch_size * added_connections
+                    leftover_num = 0
+                    batch_size = self.max_batch_size
+                else:
+                    total_batch_size = 0
+                    leftover_num = total_batch_size % added_connections
+                    batch_size = preferred_batch_size
+                    if leftover_num:
+                        batch_size -= 1
+            else:
+                batch_size = leftover_num = 0
 
             selected_sockets: List[ClientSocket] = input_sockets[:added_connections]
 
             self.clear_sockets(input_sockets)
 
-            for ind, (inputs, client_socket) in enumerate(zip(batched_inputs, selected_sockets)):
-                # logger.info('SENDING INPUTS')
-                client_socket.send_inputs(inputs, cur_ind + ind)
-                # logger.info('INPUTS SENT')
+            for ind, client_socket in enumerate(selected_sockets):
+                sample_batch_size = batch_size
+                if ind < leftover_num:
+                    sample_batch_size += 1
+                client_socket.send_inputs(self.split_batch(inputs, sample_batch_size), cur_ind + ind)
 
             cur_ind += added_connections
 
-            batched_inputs = batched_inputs[added_connections:]
-
-            if added_connections and not batched_inputs:
+            if added_connections and not total_batch_size:
                 for client_socket in client_sockets:
                     if client_socket.message_id is None:
                         client_socket.cancel_request()
 
             for client_socket in output_sockets:
-                # logger.info('RECEIVING DATA')
                 socket_outputs = client_socket.receive_outputs()
-                # logger.info('RECEIVED DATA')
                 if socket_outputs is None:
                     continue
                 batch_ind, data = socket_outputs
                 outputs[batch_ind] = data
                 received_counter += 1
-                if batched_inputs:
+                if total_batch_size:
                     client_socket.request_server()
 
-        output = self.stack_outputs(outputs)
-        # logger.info('STACKED')
-        return output
+        return self.stack_outputs(outputs)
 
     def __call__(self, **kwargs):
         return self.forward(**kwargs)
