@@ -1,7 +1,7 @@
 class Config:
     host = '127.0.0.1'
 
-    end_delimiter = b'DELIM'
+    end_delimiter = b'FINISHED'
 
     socket_timeout = 20
 
@@ -9,10 +9,10 @@ class Config:
 
     client_message_size = 65536
 
-    error_prefix = b'ERROR'
+    error_prefix = b'ERRORMSG'
 
     thread_id_len = 8
     message_id_len = 4
 
-    request_signal = b'\x00'
-    request_cancelled = b'\x01'
+    request_signal = b'POSITIVE'
+    request_cancelled = b'NEGATIVE'
