@@ -15,6 +15,7 @@ class ModelDeploymentConfig:
         self.path = path
         self.model_kwargs: Dict[str, Any] = {} if model_kwargs is None else model_kwargs
         self.port = port
-        self.allocations = list(map(int, allocations.split(','))) if type(allocations) == str else allocations
+        self.allocations: List[int] = list(
+            map(int, allocations.split(','))) if type(allocations) == str else allocations
         self.use_torchscript = use_torchscript
         self.max_batch_size = max_batch_size
